@@ -1,7 +1,7 @@
 # ABSTRACT: Utilities for Monitoring ElasticSearch
 package App::ElasticSearch::Utilities;
 
-our $VERSION = '4.7'; # VERSION
+our $VERSION = '5.0'; # VERSION
 
 use strict;
 use warnings;
@@ -15,13 +15,15 @@ our @_CONFIGS = (
     "$ENV{HOME}/.es-utils.yml",
 );
 
-use Elastijk;
 use CLI::Helpers qw(:all);
 use Getopt::Long qw(:config pass_through);
 use Hash::Flatten qw(flatten);
 use Hash::Merge::Simple qw(clone_merge);
-use Hijk;
-use JSON::XS;
+use IPC::Run3;
+use JSON::MaybeXS;
+use LWP::UserAgent;
+use Net::Netrc;
+use Ref::Util qw(is_ref is_arrayref is_hashref);
 use Time::Local;
 use URI;
 use URI::QueryParam;
@@ -30,6 +32,7 @@ use YAML;
 use Sub::Exporter -setup => {
     exports => [ qw(
         es_globals
+        es_basic_auth
         es_pattern
         es_connect
         es_request
@@ -60,6 +63,7 @@ use Sub::Exporter -setup => {
         index   => [qw(:default es_index_valid es_index_fields es_index_days_old es_index_bases)],
     },
 };
+use App::ElasticSearch::Utilities::Connection;
 use App::ElasticSearch::Utilities::VersionHacks qw(_fix_version_request);
 
 
@@ -77,6 +81,10 @@ if( !defined $_OPTIONS_PARSED ) {
         'days:i',
         'noop!',
         'datesep|date-separator:s',
+        'proto:s',
+        'http-username:s',
+        'http-password:s',
+        'password-exec:s',
     );
     $_OPTIONS_PARSED = 1;
 }
@@ -103,12 +111,21 @@ my %DEF = (
                    exists $_GLOBALS{host}         ? $_GLOBALS{host} : 'localhost',
     PORT        => exists $opt{port}              ? $opt{port} :
                    exists $_GLOBALS{port}         ? $_GLOBALS{port} : 9200,
+    PROTO       => exists $opt{proto}             ? $opt{proto} :
+                   exists $_GLOBALS{proto}        ? $_GLOBALS{proto} : 'http',
     TIMEOUT     => exists $opt{timeout}           ? $opt{timeout} :
                    exists $_GLOBALS{timeout}      ? $_GLOBALS{timeout} : 30,
     NOOP        => exists $opt{noop}              ? $opt{noop} :
                    exists $_GLOBALS{noop}         ? $_GLOBALS{noop} : undef,
     NOPROXY     => exists $opt{'keep-proxy'}      ? 0 :
                    exists $_GLOBALS{'keep-proxy'} ? $_GLOBALS{'keep-proxy'} : 1,
+    # HTTP Basic Authentication
+    USERNAME    => exists $opt{'http-username'}       ? $opt{'http-username'} :
+                   exists $_GLOBALS{'http-username'}  ? $_GLOBALS{'http-username'} : undef,
+    PASSWORD    => exists $opt{'http-password'}       ? $opt{'http-password'} :
+                   exists $_GLOBALS{'http-password'}  ? $_GLOBALS{'http-password'} : undef,
+    PASSEXEC    => exists $opt{'password-exec'}       ? $opt{'password-exec'} :
+                   exists $_GLOBALS{'password-exec'}  ? $_GLOBALS{'password-exec'} : undef,
     # Index selection options
     INDEX       => exists $opt{index}     ? $opt{index} : undef,
     BASE        => exists $opt{base}      ? lc $opt{base} :
@@ -124,6 +141,8 @@ my %DEF = (
 debug_var(\%DEF);
 CLI::Helpers::override(verbose => 1) if $DEF{NOOP};
 
+my $BASE_URL = URI->new(sprintf "%s://%s:%d", @DEF{qw(PROTO HOST PORT)});
+
 if( $DEF{NOPROXY} ) {
     debug("Removing any active HTTP Proxies from ENV.");
     delete $ENV{$_} for qw(http_proxy HTTP_PROXY);
@@ -158,6 +177,74 @@ sub es_globals {
 }
 
 
+my %_auth_cache = ();
+
+sub es_basic_auth {
+    my ($host) = @_;
+
+    $host ||= $DEF{HOST};
+
+    # Return the results if we've done this already
+    return @{ $_auth_cache{$host} }{qw(username password)}
+        if exists $_auth_cache{$host};
+
+    # Set the cached element
+    my %auth = ();
+
+    # Lookup the details netrc
+    my $netrc = Net::Netrc->lookup($host);
+    if( $DEF{HOST} eq $host ) {
+        %auth = map { lc($_) => $DEF{$_} } qw(USERNAME PASSWORD);
+    }
+
+    # Get the Username
+    $auth{username} ||= defined $DEF{USERNAME} ? $DEF{USERNAME}
+                      : defined $netrc         ? $netrc->login
+                      : prompt("Username for '$host': ",
+                            defined $DEF{USERNAME} ? (default => $DEF{USERNAME}) : ()
+                        );
+
+    # Prompt for the password
+    $auth{password} ||= defined $netrc ? $netrc->password
+                      : (es_pass_exec($host,$auth{username})
+                            || prompt(sprintf "Password for '%s' at '%s': ", $auth{username}, $host)
+                        );
+
+    # Store
+    $_auth_cache{$host} = \%auth;
+    return @auth{qw(username password)};
+}
+
+
+sub es_pass_exec {
+    my ($host,$username) = @_;
+    # Simplest case we can't run
+    return unless $DEF{PASSEXEC} and -x $DEF{PASSEXEC};
+
+    my(@out,@err);
+    # Run the password command captue out, error and RC
+    run3 [ $DEF{PASSEXEC}, $host, $username ], \undef, \@out, \@err;
+    my $rc = $?;
+
+    # Record the error
+    if( @err or $rc != 0 ) {
+        output({color=>'red',stderr=>1},
+            sprintf("es_pass_exec() called '%s' and met with an error code '%d'", $DEF{PASSEXEC}, $rc),
+            @err
+        );
+        return;
+    }
+
+    # Format and return the result
+    my $passwd = $out[-1];
+    chomp($passwd);
+
+    return unless defined $passwd and length $passwd;
+    return $passwd;
+}
+
+
+
 my %_pattern=(
     re     => $PATTERN,
     string => $DEF{PATTERN},
@@ -168,19 +255,16 @@ sub es_pattern {
 
 sub _get_es_version {
     return $CURRENT_VERSION if defined $CURRENT_VERSION;
-    eval {
-        my ($status,$result) = Elastijk::request({
-            host    => $DEF{HOST},
-            port    => $DEF{PORT},
-            method  => 'GET',
-            command => '/',
-        });
-        if( $status eq "200" ) {
-            $CURRENT_VERSION = join('.', (split /\./,$result->{version}{number})[0,1]);
+    my $conn = es_connect();
+    my $resp = $conn->ua->get( $BASE_URL->as_string );
+    if( $resp->is_success ) {
+        eval {
+            $CURRENT_VERSION = join('.', (split /\./,$resp->content->{version}{number})[0,1]);
         }
     };
     if( !defined $CURRENT_VERSION || $CURRENT_VERSION <= 0 ) {
-        output({color=>'red',stderr=>1}, "Unable to determine Elasticsearch version, something has gone terribly wrong: aborting.");
+        output({color=>'red',stderr=>1}, sprintf "[%d] Unable to determine Elasticsearch version, something has gone terribly wrong: aborting.", $resp->code);
+        output({color=>'red',stderr=>1}, $resp->content) if $resp->content;
         exit 1;
     }
     debug({color=>'magenta'}, "FOUND VERISON '$CURRENT_VERSION'");
@@ -195,6 +279,7 @@ sub es_connect {
 
     my $server = $DEF{HOST};
     my $port   = $DEF{PORT};
+    my $proto  = $DEF{PROTO};
 
     # If we're overriding, return a unique handle
     if(defined $override_servers) {
@@ -208,14 +293,15 @@ sub es_connect {
 
         if( @servers > 0 ) {
             my $pick = @servers > 1 ? $servers[int(rand(@servers))] : $servers[0];
-            return Elastijk->new(%{$pick});
+            return App::ElasticSearch::Utilities::Connection->new(%{$pick});
         }
     }
 
     # Otherwise, cache our handle
-    $ES ||= Elastijk->new(
-        host => $server,
-        port => $port
+    $ES ||= App::ElasticSearch::Utilities::Connection->new(
+        host  => $server,
+        port  => $port,
+        proto => $proto,
     );
 
     return $ES;
@@ -223,23 +309,20 @@ sub es_connect {
 
 
 sub es_request {
-    my $instance = ref $_[0] eq 'Elastijk::oo' ? shift @_ : es_connect();
+    my $instance = ref $_[0] eq 'App::ElasticSearch::Utilities::Connection' ? shift @_ : es_connect();
 
     $CURRENT_VERSION = _get_es_version() if !defined $CURRENT_VERSION;
 
     my($url,$options,$body) = _fix_version_request(@_);
 
-    # Pull connection options
-    $options->{$_} = $instance->{$_} for qw(host port);
-
+    # Normalize the options
     $options->{method} ||= 'GET';
-    $options->{body} = $body if defined $body;
     $options->{command} = $url;
-    my $index = 'NoIndex';
+    my $index;
 
     if( exists $options->{index} ) {
         my $index_in = delete $options->{index};
-        #
+
         # No need to validate _all
         if( $index_in eq '_all') {
             $index = $index_in;
@@ -254,7 +337,8 @@ sub es_request {
             $index = join(',', @valid);
         }
     }
-    $options->{index} = $index if $index ne 'NoIndex';
+    $options->{index} = $index if defined $index;
+    $index ||= '';
 
     # Figure out if we're modifying things
     my $modification = $url eq '_search' && $options->{method} eq 'POST' ? 0
@@ -262,50 +346,36 @@ sub es_request {
 
     my ($status,$res);
     if( $DEF{NOOP} && $modification) {
-        output({color=>'cyan'}, "Called es_request($index / $options->{command}), but --noop and method is $options->{method}");
+        output({color=>'cyan'}, "Called es_request($index/$options->{command}), but --noop and method is $options->{method}");
         return;
     }
-    # _cat Queries don't work with Elastijk
-    if( $url =~ m#^_cat/# ) {
-        my $uri = URI->new(sprintf "http://%s:%d/%s", $options->{host}, $options->{port}, $url);
-        # Merge URI keys and uri_param keys
-        if ( exists $options->{uri_param} ) {
-            foreach my $k (keys %{ $options->{uri_param} }) {
-                $uri->query_param( $k => $options->{uri_param}{$k} );
-            }
-        }
-        eval {
-            my $resp = Hijk::request({
-                host            => $uri->host,
-                port            => $uri->port,
-                method          => 'GET',
-                path            => $uri->path,
-                query_string    => $uri->query,
-                connect_timeout => $DEF{timeout},
-                read_timeout    => $DEF{timeout},
-            });
-            $status = $resp->{status};
-            die sprintf "%d - %s", $resp->{error_number}, $resp->{error_string} unless $status == 200;
-            $res = [ map { s/^\s+//; s/\s+$//; $_ } grep { defined && length } split /\r?\n/, $resp->{body} ];
-            1;
-        } or do {
-            output({color=>'red',stderr=>1},sprintf "es_request() hijk::request(%s?%s) failed: %s", $uri->path, $uri->query, $@);
-        };
+
+    # Make the request
+    my $resp = $instance->request($url,$options,$body);
+
+    # Check the response is defined, bail if it's not
+    die "Unsupported request method: $options->{method}" unless defined $resp;
+
+    # Logging
+    if( !$resp->is_success ) {
+        output({color=>'red',stderr=>1},
+            sprintf "es_request(%s/%s) failed[%d]: %s",
+                $index, $options->{command}, $resp->code, $resp->message
+        );
+    } elsif( !defined $resp->content || ( !is_ref($resp->content) && !length $resp->content )) {
+        output({color=>'yellow',stderr=>1},
+            sprintf "es_request(%s/%s) empty response[%d]: %s",
+                $index, $options->{command}, $resp->code, $resp->message
+        );
     }
     else {
-        eval {
-            debug("calling es_request($index / $options->{command})");
-            ($status,$res) = Elastijk::request($options);
-            die sprintf "[%d] Empty response from %s:%d", $status, $options->{host}, $options->{port} unless defined $res;
-            1;
-        } or do {
-            my $err = $@;
-            output({color=>'red',stderr=>1}, "es_request($index / $options->{command}) failed[$status]: $err");
-        };
+        debug_var($resp);
     }
-    verbose({color=>'yellow'},"es_request($index / $options->{command}) returned HTTP Status $status") if $status != 200;
+    verbose({color=>'yellow'}, sprintf "es_request(%s/%s) returned HTTP Status %s",
+        $index, $options->{command}, $resp->message,
+    ) if $resp->code != 200;
 
-    return $res;
+    return $resp->content;
 }
 
 
@@ -362,9 +432,9 @@ sub es_indices {
         push @indices, $DEF{INDEX} if es_index_valid( $DEF{INDEX} );
     }
     else {
-        my $res = es_request('_cat/indices?h=index,status');
-        foreach my $line (@{ $res }) {
-            my ($index,$status) = split /\s+/, $line;
+        my $res = es_request('_cat/indices', { uri_param => { h => 'index,status' } });
+        foreach my $entry (@{ $res }) {
+            my ($index,$status) = @{ $entry }{qw(index status)};
             debug("Evaluating '$index'");
             if(!exists $args{_all}) {
                 # State Check Disqualification
@@ -495,7 +565,7 @@ sub es_index_shards {
 
     my %shards = map { $_ => 0 } qw(primaries replicas);
     my $result = es_request('_settings', {index=>$index});
-    if( defined $result && ref $result eq 'HASH')  {
+    if( defined $result && is_hashref($result) )   {
         $shards{primaries} = $CURRENT_VERSION < 1.0 ? $result->{$index}{settings}{'index.number_of_shards'}
                                                     : $result->{$index}{settings}{index}{number_of_shards};
         $shards{replicas}  = $CURRENT_VERSION < 1.0 ? $result->{$index}{settings}{'index.number_of_replicas'}
@@ -513,7 +583,6 @@ sub es_index_valid {
     return $_valid_index{$index} if exists $_valid_index{$index};
 
     my $es = es_connect();
-
     my $result;
     eval {
         debug("Running index_exists");
@@ -576,7 +645,7 @@ sub _find_fields {
     my ($f,$ref,@path) = @_;
 
     # Handle things with properties
-    if( exists $ref->{properties} && ref $ref->{properties} eq 'HASH') {
+    if( exists $ref->{properties} && is_hashref($ref->{properties}) ) {
         foreach my $k (sort keys %{ $ref->{properties} }) {
             _find_fields($f,$ref->{properties}{$k},@path,$k);
         }
@@ -733,7 +802,7 @@ App::ElasticSearch::Utilities - Utilities for Monitoring ElasticSearch
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 SYNOPSIS
 
@@ -780,6 +849,19 @@ distribution.
 
 Grab the value of the global value from the es-utils.yaml files.
 
+=head2 es_basic_auth($host)
+
+Get the user/password combination for this host.  This is called from LWP::UserAgent if
+it recieves a 401, so the auth condition must be satisfied.
+
+Returns the username and password as a list.
+
+=head2 es_pass_exec(host, username)
+
+Called from es_basic_auth to exec a program, capture the password
+and return it to the caller.  This allows the use of password vaults
+and keychains.
+
 =head2 es_pattern
 
 Returns a hashref of the pattern filter used to get the indexes
@@ -922,6 +1004,10 @@ From App::ElasticSearch::Utilities:
     --local         Use localhost as the elasticsearch host
     --host          ElasticSearch host to connect to
     --port          HTTP port for your cluster
+    --proto         Defaults to 'http', can also be 'https'
+    --http-username HTTP Basic Auth username
+    --http-password HTTP Basic Auth password (if not specified, and --http-user is, you will be prompted)
+    --password-exec Script to run to get the users password
     --noop          Any operations other than GET are disabled, can be negated with --no-noop
     --timeout       Timeout to ElasticSearch, default 30
     --keep-proxy    Do not remove any proxy settings from %ENV
@@ -932,14 +1018,188 @@ From App::ElasticSearch::Utilities:
     --pattern       Use a pattern to operate on the indexes
     --days          If using a pattern or base, how many days back to go, default: all
 
+See also the "CONNECTION ARGUMENTS" and "INDEX SELECTION ARGUMENTS" sections from App::ElasticSearch::Utilities.
+
 =head1 ARGUMENT GLOBALS
 
 Some options may be specified in the B</etc/es-utils.yaml> or B<$HOME/.es-utils.yaml> file:
 
     ---
+    base: logstash
+    days: 7
     host: esproxy.example.com
     port: 80
     timeout: 10
+    proto: https
+    http-username: bob
+    password-exec: /home/bob/bin/get-es-passwd.sh
+
+=head1 CONNECTION ARGUMENTS
+
+Arguments for establishing a connection with the cluster.  Unless specified otherwise, these options
+can all be set in the globals file.
+
+=over
+
+=item B<local>
+
+Assume ElasticSearch is running locally, connect to localhost.
+
+=item B<host>
+
+Use a different hostname or IP address to connect.
+
+=item B<port>
+
+Defaults to 9200.
+
+=item B<proto>
+
+Defaults to 'http', can also be 'https'.
+
+=item B<http-username>
+
+If HTTP Basic Authentication is required, use this username.
+
+See also the L<HTTP Basic Authentication> section for more details
+
+=item B<http-password>
+
+If HTTP Basic Authentication is required, use this password, B<**INSECURE**>, set
+in globals, netrc, or use the B<password-exec> option below.
+
+=item B<password-exec>
+
+If HTTP Basic Authentication is required, run this command, passing the arguments:
+
+    <command_to_run> <es_host> <es_username>
+
+The script expects the last line to contain the password in plaintext.
+
+=item B<noop>
+
+Prevents any communication to the cluster from making changes to the settings or data contained therein.
+In short, it prevents anything but HEAD and GET requests, B<except> POST requests to the _search endpoint.
+
+=item B<timeout>
+
+Timeout for connections and requests, defaults to 10.
+
+=item B<keep-proxy>
+
+By default, HTTP proxy environment variables are stripped. Use this option to keep your proxy environment variables
+in tact.
+
+=back
+
+=head1 INDEX SELECTION ARGUMENTS
+
+=over
+
+=item B<base>
+
+In an environment using monthly, weekly, daily, or hourly indexes.  The base index name is everything without the date.
+Parsing for bases, also provides splitting and matching on segments of the index name delineated by the '-' character.
+If we have the following indexes:
+
+    web-dc1-YYYY.MM.DD
+    web-dc2-YYYY.MM.DD
+    logstash-dc1-YYYY.MM.DD
+    logstash-dc2-YYYY.MM.DD
+
+Valid bases would be:
+
+    web
+    web-dc1
+    web-dc2
+    logstash
+    logstash-dc1
+    logstash-dc2
+    dc1
+    dc2
+
+Combining that with the days option can provide a way to select many indexes at once.
+
+=item B<days>
+
+How many days backwards you want your operation to be relevant.
+
+=item B<datesep>
+
+Default is '.' Can be set to an empty string for no separator.
+
+=item B<pattern>
+
+A pattern to match the indexes.  Can expand the following key words and characters:
+
+    '*'    expanded to '.*'
+    'ANY'  expanded to '.*'
+    'DATE' expanded to a pattern to match a date, based on datesep
+
+The indexes are compared against this pattern.
+
+=back
+
+=head1 HTTP Basic Authentication
+
+The implementation for HTTP Basic Authentication leverages the LWP::UserAgent's underlying HTTP 401
+detection and is automatic.  There is no way to force basic authentication, it has to be requested
+by the server.  If the server does request it, here's what you need to know about how usernames and
+passwords are resolved.
+
+The username is selected by going through these mechanisms until one is found:
+
+    --http-username
+    'http-username' in /etc/es-utils.yml or ~/.es-utils.yml
+    Netrc element matching the hostname of the request
+    CLI::Helpers prompt()
+
+Once the username has been resolved, the following mechanisms are tried in order:
+
+    --http-password
+    'http-password' in /etc/es-utils.yml or ~/.es-utils.yml
+    Netrc element matching the hostname of the request
+    Password executable defined by --password-exec
+    'password-exec' in /etc/es-utils.yml, ~/.es-utils.yml
+    CLI::Helpers prompt()
+
+=head2 Password Exec
+
+It is B<BAD> practice to specify passwords as a command line argument, or store it in a plaintext
+file.  There are cases where this may be necessary, but it is not recommended.  The best method for securing  your
+password is to use the B<password-exec> option.
+
+This option must point to an executable script.  That script will be passed two arguments, the hostname and the username
+for the request.  It expects the password printed to STDOUT as the last line of output.  Here's an example password-exec setup
+using Apple Keychain:
+
+    #!/bin/sh
+
+    HOSTNAME=$1;
+    USERNAME=$2;
+
+    /usr/bin/security find-generic-password -w -a "$USERNAME" -s "$HOSTNAME"
+
+If we save this to "$HOME/bin/get-passwd.sh" we can execute a script
+like this:
+
+    $ es-search.pl --http-username bob --password-exec $HOME/bin/get-passwd.sh \
+                    --base secure-data --fields
+
+Though it's probably best to set this in your ~/.es-utils.yml file:
+
+    ---
+    host: secured-cluster.example.org
+    port: 443
+    proto: https
+    http-username: bob
+    password-exec: /home/bob/bin/get-passwd.sh
+
+=head3 CLI::Helpers and Password Prompting
+
+If all the fails to yield a password, the last resort is to use CLI::Helpers::prompt() to ask the user for their
+password.  If the user is using version 1.1 or higher of CLI::Helpers, this call will turn off echo and readline magic
+for the password prompt.
 
 =head1 INSTALL
 
