@@ -1,7 +1,7 @@
 # ABSTRACT: Fix version issues to support all the things
 package App::ElasticSearch::Utilities::VersionHacks;
 
-our $VERSION = '4.7'; # VERSION
+our $VERSION = '5.0'; # VERSION
 use strict;
 use warnings;
 
@@ -21,18 +21,26 @@ const my %SIMPLE => (
     '_optimize' => {
         # Yes, in case you're wondering _optimize disappeared in 2.2 after being deprecated in 2.1
         default => '_forcemerge',
-        1.0 => '_optimize',
-        1.1 => '_optimize',
-        1.2 => '_optimize',
-        1.3 => '_optimize',
-        1.4 => '_optimize',
-        1.5 => '_optimize',
-        1.6 => '_optimize',
-        1.7 => '_optimize',
-        1.8 => '_optimize',
-        1.9 => '_optimize',
-        2.0 => '_optimize',
+        'v1.0'  => '_optimize',
+        'v1.1'  => '_optimize',
+        'v1.2'  => '_optimize',
+        'v1.3'  => '_optimize',
+        'v1.4'  => '_optimize',
+        'v1.5'  => '_optimize',
+        'v1.6'  => '_optimize',
+        'v1.7'  => '_optimize',
+        'v1.8'  => '_optimize',
+        'v1.9'  => '_optimize',
+        'v2.0'  => '_optimize',
     },
+    '_status' => {
+        default => '_status',
+        'v2.0'  => '_stats',
+        'v2.1'  => '_stats',
+        'v2.2'  => '_stats',
+        'v2.3'  => '_stats',
+        'v5.0'  => '_stats',
+    }
 );
 my %CALLBACKS = (
     '_cluster/state' => {
@@ -48,26 +56,27 @@ sub _fix_version_request {
     # Requires App::ElasticSearch::Utilities to be loaded
     if( ! defined $version  ){
         eval {
-            $version = App::ElasticSearch::Utilities::_get_es_version();
-        };
-        if(my $err = $@) {
+            $version = sprintf "%0.1f", App::ElasticSearch::Utilities::_get_es_version();
+            1;
+        } or do {
+            my $err = $@;
             output({stderr=>1,color=>'red'}, "Failed version detection!", $@);
-        }
-        if (defined $version < $MIN_VERSION) {
+        };
+        if (defined $version && $version < $MIN_VERSION) {
             output({stderr=>1,color=>'red',sticky=>1},
                     "!!! Detected ElasticSearch Version '$version', which is < $MIN_VERSION, please upgrade your cluster !!!");
             exit 1;
         }
     }
 
-    return @_ unless defined $version;
+    my $vstr = sprintf "v%0.1f", $version;
 
     if(exists $SIMPLE{$url}) {
         my $versions = join(", ", sort keys %{ $SIMPLE{$url} });
-        debug("Method changed in API, evaluating rewrite ($versions) against $version");
-        if(exists $SIMPLE{$url}->{$version}) {
-            debug({indent=>1,color=>'yellow'}, "+ Rewriting $url to $SIMPLE{$url}->{$version}");
-            $url = $SIMPLE{$url}->{$version};
+        debug("Method changed in API, evaluating rewrite ($versions) against $vstr");
+        if(exists $SIMPLE{$url}->{$vstr}) {
+            debug({indent=>1,color=>'yellow'}, "+ Rewriting $url to $SIMPLE{$url}->{$vstr}");
+            $url = $SIMPLE{$url}->{$vstr};
         }
         elsif(exists $SIMPLE{$url}->{default}) {
             debug({indent=>1,color=>'yellow'}, "+ Rewriting $url to $SIMPLE{$url}->{default} by default rule");
@@ -83,10 +92,10 @@ sub _fix_version_request {
         }
         if( defined $cb ) {
             my $versions = join(", ", sort keys %{ $CALLBACKS{$cb} });
-            debug("Method changed in API, evaluating callback for $cb ($versions) against $version");
-            if(exists $CALLBACKS{$url}->{$version}) {
+            debug("Method changed in API, evaluating callback for $cb ($versions) against $vstr");
+            if(exists $CALLBACKS{$url}->{$vstr}) {
                 debug({indent=>1,color=>'yellow'}, "+ Callback dispatched for $url");
-                ($url,$options,$data) = $CALLBACKS{$url}->{$version}->($url,$options,$data);
+                ($url,$options,$data) = $CALLBACKS{$url}->{$vstr}->($url,$options,$data);
             }
             elsif(exists $CALLBACKS{$url}->{default}) {
                 debug({indent=>1,color=>'yellow'}, "+ Callback dispatched for $url by default rule");
@@ -157,7 +166,7 @@ App::ElasticSearch::Utilities::VersionHacks - Fix version issues to support all
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 AUTHOR
 
