@@ -4,11 +4,11 @@
 use strict;
 use warnings;
 
-use JSON::XS;
+use App::ElasticSearch::Utilities qw(:default es_apply_index_settings es_open_index es_close_index);
+use CLI::Helpers qw(:all);
+use JSON::MaybeXS;
 use Getopt::Long qw(:config no_ignore_case no_ignore_case_always);
 use Pod::Usage;
-use CLI::Helpers qw(:all);
-use App::ElasticSearch::Utilities qw(:default es_apply_index_settings es_open_index es_close_index);
 
 #------------------------------------------------------------------------#
 # Argument Collection
@@ -103,7 +103,7 @@ es-apply-settings.pl - Run to apply a JSON list of settings to indexes matching
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 SYNOPSIS
 
@@ -120,6 +120,10 @@ From App::ElasticSearch::Utilities:
     --local         Use localhost as the elasticsearch host
     --host          ElasticSearch host to connect to
     --port          HTTP port for your cluster
+    --proto         Defaults to 'http', can also be 'https'
+    --http-username HTTP Basic Auth username
+    --http-password HTTP Basic Auth password (if not specified, and --http-user is, you will be prompted)
+    --password-exec Script to run to get the users password
     --noop          Any operations other than GET are disabled, can be negated with --no-noop
     --timeout       Timeout to ElasticSearch, default 30
     --keep-proxy    Do not remove any proxy settings from %ENV
@@ -130,6 +134,8 @@ From App::ElasticSearch::Utilities:
     --pattern       Use a pattern to operate on the indexes
     --days          If using a pattern or base, how many days back to go, default: all
 
+See also the "CONNECTION ARGUMENTS" and "INDEX SELECTION ARGUMENTS" sections from App::ElasticSearch::Utilities.
+
 From CLI::Helpers:
 
     --data-file         Path to a file to write lines tagged with 'data => 1'
