@@ -13,7 +13,7 @@ use File::Basename;
 use File::Slurp::Tiny qw(read_lines);
 use Getopt::Long qw(:config posix_default no_ignore_case no_ignore_case_always);
 use Hash::Merge::Simple qw(clone_merge);
-use JSON::XS;
+use JSON::MaybeXS;
 use Pod::Usage;
 use Time::HiRes qw(time);
 
@@ -59,7 +59,7 @@ if( $HOST{to} eq $HOST{from} && $INDEX{to} eq $INDEX{from} ) {
 
 #------------------------------------------------------------------------#
 # Build the Query
-my $JSON = JSON::XS->new->pretty->canonical;
+my $JSON = JSON->new->pretty->canonical;
 my $qs = App::ElasticSearch::Utilities::QueryString->new();
 my $q  = @ARGV ? $qs->expand_query_string(@ARGV)
                : App::ElasticSearch::Utilities::Query->new(must => {match_all=>{}});
@@ -118,20 +118,22 @@ unless( exists $OPT{append} ) {
         };
     }
     else {
-        $res = es_request($ES{from}, '_mappings', {index => $INDEX{from}});
         $mappings = $res->{$INDEX{from}}{mappings};
     }
 
-    ($status, $res) = $ES{to}->put(
-        index => $INDEX{to},
-        body  => {
+    $res = es_request('/',
+        {
+            method => 'PUT',
+            index => $INDEX{to},
+        },
+        {
             settings => $to_settings,
             mappings => $mappings,
         }
     );
 
-    if ($status ne "200") {
-        die "Failed to create index in $HOST{to} (http status = $status): " . $JSON->encode([ $status, $res ]);
+    if (!defined $res || !is_hashref($res) || !$res->{ok}) {
+        die "Failed to create index in $HOST{to} : " . $JSON->encode($res);
     }
 }
 else {
@@ -226,7 +228,7 @@ es-copy-index.pl - Copy an index from one cluster to another
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 SYNOPSIS
 
@@ -250,6 +252,10 @@ From App::ElasticSearch::Utilities:
     --local         Use localhost as the elasticsearch host
     --host          ElasticSearch host to connect to
     --port          HTTP port for your cluster
+    --proto         Defaults to 'http', can also be 'https'
+    --http-username HTTP Basic Auth username
+    --http-password HTTP Basic Auth password (if not specified, and --http-user is, you will be prompted)
+    --password-exec Script to run to get the users password
     --noop          Any operations other than GET are disabled, can be negated with --no-noop
     --timeout       Timeout to ElasticSearch, default 30
     --keep-proxy    Do not remove any proxy settings from %ENV
@@ -260,6 +266,8 @@ From App::ElasticSearch::Utilities:
     --pattern       Use a pattern to operate on the indexes
     --days          If using a pattern or base, how many days back to go, default: all
 
+See also the "CONNECTION ARGUMENTS" and "INDEX SELECTION ARGUMENTS" sections from App::ElasticSearch::Utilities.
+
 From CLI::Helpers:
 
     --data-file         Path to a file to write lines tagged with 'data => 1'
