@@ -21,6 +21,8 @@ GetOptions(\%opt,
     'carbon-port:i',
     'with-indices',
     'with-cluster',
+    'prefix:s',
+    'no-prefix',
     'help|h',
     'manual|m',
 );
@@ -42,7 +44,7 @@ my %cfg = (
     'carbon-proto' => 'tcp',
     'carbon-base'  => 'general.es',
     %opt,
-    host => App::ElasticSearch::Utilities::def('HOST')
+    host => App::ElasticSearch::Utilities::def('HOST'),
 );
 
 #------------------------------------------------------------------------#
@@ -90,7 +92,8 @@ if( exists $cfg{'with-indices'} ) {
 # Send output to appropriate channels
 for ( @metrics ) {
     # Format
-    s/^/$cfg{'carbon-base'}.$HOSTNAME./;
+    my $prefix = exists $cfg{prefix} ? $cfg{prefix} : join('.', $cfg{'carbon-base'}, $HOSTNAME);
+    s/^/$prefix./ unless $cfg{'no-prefix'};
     s/$/ $TIME\n/;
 
     # Send the Data
@@ -192,7 +195,7 @@ es-graphite-dynamic.pl - Dynamically gather metrics and send to graphite
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 SYNOPSIS
 
@@ -208,12 +211,18 @@ Options:
     --carbon-proto      Protocol for to use for Carbon (Default: tcp)
     --with-indices      Also send individual index stats
     --ignore            Comma separated list of keys to ignore in collection
+    --prefix            A metric path to prefix stats, defaults to (--carbon-base).(hostname)
+    --no-prefix         Don't prefix the metrics at all
 
 From App::ElasticSearch::Utilities:
 
     --local         Use localhost as the elasticsearch host
     --host          ElasticSearch host to connect to
     --port          HTTP port for your cluster
+    --proto         Defaults to 'http', can also be 'https'
+    --http-username HTTP Basic Auth username
+    --http-password HTTP Basic Auth password (if not specified, and --http-user is, you will be prompted)
+    --password-exec Script to run to get the users password
     --noop          Any operations other than GET are disabled, can be negated with --no-noop
     --timeout       Timeout to ElasticSearch, default 30
     --keep-proxy    Do not remove any proxy settings from %ENV
@@ -224,6 +233,8 @@ From App::ElasticSearch::Utilities:
     --pattern       Use a pattern to operate on the indexes
     --days          If using a pattern or base, how many days back to go, default: all
 
+See also the "CONNECTION ARGUMENTS" and "INDEX SELECTION ARGUMENTS" sections from App::ElasticSearch::Utilities.
+
 From CLI::Helpers:
 
     --data-file         Path to a file to write lines tagged with 'data => 1'
@@ -283,6 +294,29 @@ Examples:
 
     es-graphite-dynamic.pl --with-indices --ignore primaries,get,warmer
 
+=item B<prefix>
+
+A metric path to prefix the collected stats with.  This is useful for using es-graphite-dynamic.pl with another
+collector such as Diamond which expects metrics in a certain format.  To use with diamond's userscripts or files collector
+you could:
+
+    #!/bin/sh
+    # userscripts
+
+    es-graphite-dynamic.pl --local --prefix elasticsearch
+
+Or with the file collector, you could cron this:
+
+    es-graphite-dynamic.pl --loocal --prefix elasticsearch --data-file /tmp/diamond/elasticsearch.out --quiet
+
+If not specified, the assumption is data will be going directly to graphite and the metric path will be set as:
+
+    'general.es.$HOSTNAME'
+
+=item B<no-prefix>
+
+Don't set the prefix to the metric names.
+
 =back
 
 =head1 AUTHOR
